@@ -35,8 +35,11 @@
 #include "gda-connection.h"
 #include "gda-server-provider.h"
 #include "gda-util.h"
+#include <libgda/gda-custom-marshal.h>
+#include <libgda/gda-error.h>
 
 extern xmlDtdPtr gda_paramlist_dtd;
+extern gchar *gda_lang_locale;
 
 /* 
  * Main static functions 
@@ -50,14 +53,16 @@ static void set_remove_node (GdaSet *set, GdaSetNode *node);
 static void set_remove_source (GdaSet *set, GdaSetSource *source);
 
 static void changed_holder_cb (GdaHolder *holder, GdaSet *dataset);
+static GError *validate_change_holder_cb (GdaHolder *holder, const GValue *value, GdaSet *dataset);
 static void source_changed_holder_cb (GdaHolder *holder, GdaSet *dataset);
-static void notify_holder_cb (GdaHolder *holder, GParamSpec *pspec, GdaSet *dataset);
+static void att_holder_changed_cb (GdaHolder *holder, const gchar *att_name, const GValue *att_value, GdaSet *dataset);
 
 static void compute_public_data (GdaSet *set);
-static void gda_set_real_add_holder (GdaSet *set, GdaHolder *holder);
+static gboolean gda_set_real_add_holder (GdaSet *set, GdaHolder *holder);
 
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass  *parent_class = NULL;
+extern GdaAttributesManager *gda_holder_attributes_manager;
 
 /* properties */
 enum
@@ -74,12 +79,13 @@ enum
 {
 	HOLDER_CHANGED,
 	PUBLIC_DATA_CHANGED,
-	HOLDER_PLUGIN_CHANGED,
 	HOLDER_ATTR_CHANGED,
+	VALIDATE_HOLDER_CHANGE,
+	VALIDATE_SET,
 	LAST_SIGNAL
 };
 
-static gint gda_set_signals[LAST_SIGNAL] = { 0, 0, 0, 0 };
+static gint gda_set_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0 };
 
 
 /* private structure */
@@ -88,6 +94,7 @@ struct _GdaSetPrivate
 	gchar           *id;
 	gchar           *name;
 	gchar           *descr;
+	GHashTable      *holders_hash; /* key = GdaHoler ID, value = GdaHolder */
 };
 
 static void 
@@ -170,6 +177,7 @@ gda_set_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static const GTypeInfo info = {
 			sizeof (GdaSetClass),
 			(GBaseInitFunc) NULL,
@@ -182,12 +190,40 @@ gda_set_get_type (void)
 			(GInstanceInitFunc) gda_set_init
 		};
 		
-		type = g_type_register_static (G_TYPE_OBJECT, "GdaSet", &info, 0);
+		g_static_mutex_lock (&registering);
+		if (type == 0)
+			type = g_type_register_static (G_TYPE_OBJECT, "GdaSet", &info, 0);
+		g_static_mutex_unlock (&registering);
 	}
 
 	return type;
 }
 
+static gboolean
+validate_accumulator (GSignalInvocationHint *ihint,
+		      GValue *return_accu,
+		      const GValue *handler_return,
+		      gpointer data)
+{
+	GError *error;
+
+	error = g_value_get_boxed (handler_return);
+	g_value_set_boxed (return_accu, error);
+
+	return error ? FALSE : TRUE; /* stop signal if an error has been set */
+}
+
+static GError *
+m_validate_holder_change (GdaSet *set, GdaHolder *holder, const GValue *new_value)
+{
+	return NULL;
+}
+
+static GError *
+m_validate_set (GdaSet *set)
+{
+	return NULL;
+}
 
 static void
 gda_set_class_init (GdaSetClass *class)
@@ -197,39 +233,70 @@ gda_set_class_init (GdaSetClass *class)
 	parent_class = g_type_class_peek_parent (class);
 
 	gda_set_signals[HOLDER_CHANGED] =
-		g_signal_new ("holder_changed",
+		g_signal_new ("holder-changed",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GdaSetClass, holder_changed),
 			      NULL, NULL,
-			      gda_marshal_VOID__OBJECT, G_TYPE_NONE, 1,
+			      _gda_marshal_VOID__OBJECT, G_TYPE_NONE, 1,
 			      GDA_TYPE_HOLDER);
-	gda_set_signals[HOLDER_PLUGIN_CHANGED] =
-		g_signal_new ("holder_plugin_changed",
+
+	/**
+	 * GdaSet::validate-holder-change:
+	 * @set: the object which received the signal
+	 * @holder: the #GdaHolder which is going to change
+	 * @new_value: the proposed new value for @holder
+	 * 
+	 * Gets emitted when a #GdaHolder's in @set is going to change its value. One can connect to
+	 * this signal to control which values @holder can have (for example to implement some business rules)
+	 *
+	 * Return value: NULL if @holder is allowed to change its value to @new_value, or a #GError
+	 * otherwise.
+	 */
+	gda_set_signals[VALIDATE_HOLDER_CHANGE] =
+		g_signal_new ("validate-holder-change",
 			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (GdaSetClass, holder_plugin_changed),
-			      NULL, NULL,
-			      gda_marshal_VOID__OBJECT, G_TYPE_NONE, 1,
-			      GDA_TYPE_HOLDER);
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GdaSetClass, validate_holder_change),
+			      validate_accumulator, NULL,
+			      _gda_marshal_ERROR__OBJECT_VALUE, GDA_TYPE_ERROR, 2,
+			      GDA_TYPE_HOLDER, G_TYPE_VALUE);
+	/**
+	 * GdaSet::validate-set:
+	 * @set: the object which received the signal
+	 * 
+	 * Gets emitted when gda_set_is_valid() is called, use
+	 * this signal to control which combination of values @set's holder can have (for example to implement some business rules)
+	 *
+	 * Return value: NULL if @set's contents has been validated, or a #GError
+	 * otherwise.
+	 */
+	gda_set_signals[VALIDATE_SET] =
+		g_signal_new ("validate-set",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GdaSetClass, validate_set),
+			      validate_accumulator, NULL,
+			      _gda_marshal_ERROR__VOID, GDA_TYPE_ERROR, 0);
 	gda_set_signals[HOLDER_ATTR_CHANGED] =
-		g_signal_new ("holder_attr_changed",
+		g_signal_new ("holder-attr-changed",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GdaSetClass, holder_attr_changed),
 			      NULL, NULL,
-			      gda_marshal_VOID__OBJECT, G_TYPE_NONE, 1,
-			      GDA_TYPE_HOLDER);
+			      _gda_marshal_VOID__OBJECT_STRING_VALUE, G_TYPE_NONE, 3,
+			      GDA_TYPE_HOLDER, G_TYPE_STRING, G_TYPE_VALUE);
 	gda_set_signals[PUBLIC_DATA_CHANGED] =
-		g_signal_new ("public_data_changed",
+		g_signal_new ("public-data-changed",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GdaSetClass, public_data_changed),
 			      NULL, NULL,
-			      gda_marshal_VOID__VOID, G_TYPE_NONE, 0);
+			      _gda_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
 	class->holder_changed = NULL;
-	class->holder_plugin_changed = NULL;
+	class->validate_holder_change = m_validate_holder_change;
+	class->validate_set = m_validate_set;
 	class->holder_attr_changed = NULL;
 	class->public_data_changed = NULL;
 
@@ -262,6 +329,7 @@ gda_set_init (GdaSet *set)
 	set->nodes_list = NULL;
 	set->sources_list = NULL;
 	set->groups_list = NULL;
+	set->priv->holders_hash = g_hash_table_new (g_str_hash, g_str_equal);
 }
 
 
@@ -329,11 +397,12 @@ gda_set_copy (GdaSet *set)
 GdaSet *
 gda_set_new_inline (gint nb, ...)
 {
-	GdaSet *set;
+	GdaSet *set = NULL;
 	GSList *holders = NULL;
 	va_list ap;
 	gchar *id;
 	gint i;
+	gboolean allok = TRUE;
 
 	/* build the list of holders */
 	va_start (ap, nb);
@@ -341,10 +410,11 @@ gda_set_new_inline (gint nb, ...)
 		GType type;
 		GdaHolder *holder;
 		GValue *value;
+		GError *lerror = NULL;
 
 		id = va_arg (ap, char *);
 		type = va_arg (ap, GType);
-		holder = (GdaHolder *) g_object_new (GDA_TYPE_HOLDER, "g_type", type, "id", id, NULL);
+		holder = (GdaHolder *) g_object_new (GDA_TYPE_HOLDER, "g-type", type, "id", id, NULL);
 
 		value = gda_value_new (type);
 		if (type == G_TYPE_BOOLEAN) 
@@ -383,38 +453,52 @@ gda_set_new_inline (gint nb, ...)
 			g_value_set_long (value, va_arg (ap, glong));
 		else if (type == G_TYPE_ULONG)
 			g_value_set_ulong (value, va_arg (ap, gulong));
-		else
-			g_warning (_("%s() does not handle values of type %s, value not assigned."),
+		else if (type == G_TYPE_GTYPE)
+			g_value_set_gtype (value, va_arg(ap, GType));
+		else {
+			g_warning (_("%s() does not handle values of type '%s'."),
 				   __FUNCTION__, g_type_name (type));
+			g_object_unref (holder);
+			allok = FALSE;
+			break;
+		}
 
-		if (!gda_holder_take_value (holder, value))
-			g_warning (_("Could not set GdaHolder's value, value not assigned"));
+		if (!gda_holder_take_value (holder, value, &lerror)) {
+			g_warning (_("Unable to set holder's value: %s"),
+				   lerror && lerror->message ? lerror->message : _("No detail"));
+			if (lerror)
+				g_error_free (lerror);
+			g_object_unref (holder);
+			allok = FALSE;
+			break;
+		}
 		holders = g_slist_append (holders, holder);
         }
 	va_end (ap);
 
 	/* create the set */
-	set = gda_set_new (holders);
+	if (allok) 
+		set = gda_set_new (holders);
 	if (holders) {
 		g_slist_foreach (holders, (GFunc) g_object_unref, NULL);
 		g_slist_free (holders);
 	}
-
 	return set;
 }
 
 /**
  * gda_set_set_holder_value
  * @set: a #GdaSet object
+ * @error: a place to store errors, or %NULL
  * @holder_id: the ID of the holder to set the value
- * @...: value, of the correct type, depending on the requested holder's type
+ * @...: value, of the correct type, depending on the requested holder's type (not NULL)
  *
  * Set the value of the #GdaHolder which ID is @holder_id to a specified value
  *
  * Returns: TRUE if no error occurred and the value was set correctly
  */
 gboolean
-gda_set_set_holder_value (GdaSet *set, const gchar *holder_id, ...)
+gda_set_set_holder_value (GdaSet *set, GError **error, const gchar *holder_id, ...)
 {
 	GdaHolder *holder;
 	va_list ap;
@@ -426,7 +510,8 @@ gda_set_set_holder_value (GdaSet *set, const gchar *holder_id, ...)
 
 	holder = gda_set_get_holder (set, holder_id);
 	if (!holder) {
-		g_warning (_("GdaHolder with ID '%s' not found in set"), holder_id);
+		g_set_error (error, GDA_SET_ERROR, GDA_SET_HOLDER_NOT_FOUND_ERROR,
+			     _("GdaHolder with ID '%s' not found in set"), holder_id);
 		return FALSE;
 	}
 	type = gda_holder_get_g_type (holder);
@@ -468,16 +553,18 @@ gda_set_set_holder_value (GdaSet *set, const gchar *holder_id, ...)
 		g_value_set_long (value, va_arg (ap, glong));
 	else if (type == G_TYPE_ULONG)
 		g_value_set_ulong (value, va_arg (ap, gulong));
-	else
-		g_warning (_("%s() does not handle values of type %s, value not assigned."),
-			   __FUNCTION__, g_type_name (type));
-	
-	va_end (ap);
-	if (!gda_holder_take_value (holder, value)) {
-		g_warning (_("Could not set GdaHolder's value, value not assigned"));
+	else if (type == G_TYPE_GTYPE)
+		g_value_set_gtype (value, va_arg (ap, GType));	
+	else {
+		g_set_error (error, 0, 0,
+			     _("%s() does not handle values of type '%s'."),
+			     __FUNCTION__, g_type_name (type));
+		va_end (ap);
 		return FALSE;
 	}
-	return TRUE;
+
+	va_end (ap);
+	return gda_holder_take_value (holder, value, error);
 }
 
 /**
@@ -588,7 +675,7 @@ gda_set_new_from_spec_string (const gchar *xml_spec, GError **error)
                                 g_set_error (error,
                                              GDA_SET_ERROR,
                                              GDA_SET_XML_SPEC_ERROR,
-                                             "XML spec. does not conform to DTD");
+                                             "%s", "XML spec. does not conform to DTD");
 
                         xmlDoValidityCheckingDefaultValue = xmlcheck;
                         return NULL;
@@ -635,18 +722,12 @@ gda_set_new_from_spec_node (xmlNodePtr xml_spec, GError **error)
 	GdaSet *set = NULL;
 	GSList *holders = NULL, *sources = NULL;
 	GSList *list;
-	const gchar *lang;
+	const gchar *lang = gda_lang_locale;
 
 	xmlNodePtr cur;
 	gboolean allok = TRUE;
 	gchar *str;
 
-#ifdef HAVE_LC_MESSAGES
-	lang = setlocale (LC_MESSAGES, NULL);
-#else
-	lang = setlocale (LC_CTYPE, NULL);
-#endif
-
 	if (strcmp ((gchar*)xml_spec->name, "parameters") != 0){
 		g_set_error (error, GDA_SET_ERROR, GDA_SET_XML_SPEC_ERROR,
 			     _("Missing node <parameters>: '%s'"), xml_spec->name);
@@ -670,7 +751,7 @@ gda_set_new_from_spec_node (xmlNodePtr xml_spec, GError **error)
 				errors = gda_data_model_import_get_errors (GDA_DATA_MODEL_IMPORT (model));
 				if (errors) {
 					GError *err = (GError *) errors->data;
-					g_set_error (error, 0, 0, err->message);
+					g_set_error (error, 0, 0, "%s", err->message);
 					g_object_unref (model);
 					model = NULL;
 					allok = FALSE;
@@ -724,33 +805,33 @@ gda_set_new_from_spec_node (xmlNodePtr xml_spec, GError **error)
 			gdatype = xmlGetProp (cur, BAD_CAST "gdatype");
 
 			if (!holder) {
-				holder = GDA_HOLDER (g_object_new (GDA_TYPE_HOLDER,
-								   "g_type", 
-								   gdatype ? gda_g_type_from_string ((gchar *) gdatype) : G_TYPE_STRING,
-								   NULL));
+				holder = (GdaHolder*) (g_object_new (GDA_TYPE_HOLDER,
+								     "g-type", 
+								     gdatype ? gda_g_type_from_string ((gchar *) gdatype) : G_TYPE_STRING,
+								     NULL));
 				holders = g_slist_append (holders, holder);
 			}
-			if (gdatype) xmlFree (gdatype);
+			if (gdatype)
+				xmlFree (gdatype);
 			
 			/* set holder's attributes */
-			gda_utility_holder_load_attributes (holder, cur, sources);
+			if (! gda_utility_holder_load_attributes (holder, cur, sources, error))
+				allok = FALSE;
 		}
 	}
 
 	/* setting prepared new names from sources (models) */
-	list = sources;
-	while (list) {
+	for (list = sources; list; list = list->next) {
 		str = g_object_get_data (G_OBJECT (list->data), "newname");
 		if (str) {
-			g_object_set_data_full (G_OBJECT (list->data), "name", str, g_free);
+			g_object_set_data_full (G_OBJECT (list->data), "name", g_strdup (str), g_free);
 			g_object_set_data (G_OBJECT (list->data), "newname", NULL);
 		}
 		str = g_object_get_data (G_OBJECT (list->data), "newdescr");
 		if (str) {
-			g_object_set_data_full (G_OBJECT (list->data), "descr", str, g_free);
+			g_object_set_data_full (G_OBJECT (list->data), "descr", g_strdup (str), g_free);
 			g_object_set_data (G_OBJECT (list->data), "newdescr", NULL);
 		}
-		list = g_slist_next (list);
 	}
 
 	/* holders' values, constraints: TODO */
@@ -785,71 +866,6 @@ gda_set_new_from_spec_node (xmlNodePtr xml_spec, GError **error)
 	return set;
 }
 
-
-/**
- * gda_set_get_spec
- * @set: a #GdaSet object
- *
- * Get the specification as an XML string. See the gda_set_new_from_spec_string()
- * form more information about the XML specification string format.
- *
- * Returns: a new string
- */
-gchar *
-gda_set_get_spec (GdaSet *set)
-{
-	xmlDocPtr doc;
-	xmlNodePtr root;
-	xmlChar *xmlbuff;
-	int buffersize;
-	GSList *list;
-
-	g_return_val_if_fail (GDA_IS_SET (set), NULL);
-
-	doc = xmlNewDoc ((xmlChar*)"1.0");
-	g_return_val_if_fail (doc, NULL);
-	root = xmlNewDocNode (doc, NULL, (xmlChar*)"data-set-spec", NULL);
-	xmlDocSetRootElement (doc, root);
-
-	/* holders list */
-	for (list = set->holders; list; list = list->next) {
-		xmlNodePtr node;
-		GdaHolder *holder = GDA_HOLDER (list->data);
-		gchar *str;
-		const gchar *cstr;
-
-		node = xmlNewTextChild (root, NULL, (xmlChar*)"parameter", NULL);
-		g_object_get (G_OBJECT (holder), "id", &str, NULL);
-		if (str) 
-			xmlSetProp(node, (xmlChar*)"id", (xmlChar*)str);
-		g_free (str);
-
-		g_object_get (G_OBJECT (holder), "name", &str, NULL);
-		if (str)
-			xmlSetProp(node, (xmlChar*)"name", (xmlChar*)cstr);
-		g_free (str);
-
-		g_object_get (G_OBJECT (holder), "description", &str, NULL);
-		if (str)
-			xmlSetProp(node, (xmlChar*)"descr", (xmlChar*)str);
-		g_free (str);
-
-		xmlSetProp(node, (xmlChar*)"gdatype", (xmlChar*)gda_g_type_to_string (gda_holder_get_g_type (holder)));
-
-		xmlSetProp(node, (xmlChar*)"nullok", (xmlChar*)(gda_holder_get_not_null (holder) ? "FALSE" : "TRUE"));
-		str = g_object_get_data (G_OBJECT (holder), "__gda_entry_plugin");
-		if (str) 
-			xmlSetProp(node, (xmlChar*)"plugin", (xmlChar*)str);
-	}
-
-	/* holders' values, sources, constraints: TODO */
-
-	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
-	
-	xmlFreeDoc(doc);
-	return (gchar *) xmlbuff;
-}
-
 /**
  * gda_set_remove_holder
  * @set:
@@ -866,10 +882,12 @@ gda_set_remove_holder (GdaSet *set, GdaHolder *holder)
 
 	g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
 					      G_CALLBACK (changed_holder_cb), set);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
+					      G_CALLBACK (validate_change_holder_cb), set);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
 					      G_CALLBACK (source_changed_holder_cb), set);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
-					      G_CALLBACK (notify_holder_cb), set);
+					      G_CALLBACK (att_holder_changed_cb), set);
 
 	/* now destroy the GdaSetNode and the GdaSetSource if necessary */
 	node = gda_set_get_node (set, holder);
@@ -886,6 +904,7 @@ gda_set_remove_holder (GdaSet *set, GdaHolder *holder)
 	set_remove_node (set, node);
 
 	set->holders = g_slist_remove (set->holders, holder);
+	g_hash_table_remove (set->priv->holders_hash, gda_holder_get_id (holder));
 	g_object_unref (G_OBJECT (holder));
 }
 
@@ -896,26 +915,30 @@ source_changed_holder_cb (GdaHolder *holder, GdaSet *set)
 }
 
 static void
-notify_holder_cb (GdaHolder *holder, GParamSpec *pspec, GdaSet *set)
+att_holder_changed_cb (GdaHolder *holder, const gchar *att_name, const GValue *att_value, GdaSet *set)
 {
-	if (!strcmp (pspec->name, "entry-plugin")) {
 #ifdef GDA_DEBUG_signal
-		g_print (">> 'HOLDER_PLUGIN_CHANGED' from %s\n", __FUNCTION__);
+	g_print (">> 'HOLDER_ATTR_CHANGED' from %s\n", __FUNCTION__);
 #endif
-		g_signal_emit (G_OBJECT (set), gda_set_signals[HOLDER_PLUGIN_CHANGED], 0, holder);
+	g_signal_emit (G_OBJECT (set), gda_set_signals[HOLDER_ATTR_CHANGED], 0, holder, att_name, att_value);
 #ifdef GDA_DEBUG_signal
-		g_print ("<< 'HOLDER_PLUGIN_CHANGED' from %s\n", __FUNCTION__);
+	g_print ("<< 'HOLDER_ATTR_CHANGED' from %s\n", __FUNCTION__);
 #endif
-	}
-	if (!strcmp (pspec->name, "use-default-value")) {
+}
+
+static GError *
+validate_change_holder_cb (GdaHolder *holder, const GValue *value, GdaSet *set)
+{
+	/* signal the holder validate-change */
+	GError *error = NULL;
 #ifdef GDA_DEBUG_signal
-		g_print (">> 'HOLDER_ATTR_CHANGED' from %s\n", __FUNCTION__);
+	g_print (">> 'VALIDATE_HOLDER_CHANGE' from %s\n", __FUNCTION__);
 #endif
-		g_signal_emit (G_OBJECT (set), gda_set_signals[HOLDER_ATTR_CHANGED], 0, holder);
+	g_signal_emit (G_OBJECT (set), gda_set_signals[VALIDATE_HOLDER_CHANGE], 0, holder, value, &error);
 #ifdef GDA_DEBUG_signal
-		g_print ("<< 'HOLDER_ATTR_CHANGED' from %s\n", __FUNCTION__);
+	g_print ("<< 'VALIDATE_HOLDER_CHANGED' from %s\n", __FUNCTION__);
 #endif
-	}
+	return error;
 }
 
 static void
@@ -952,15 +975,21 @@ gda_set_dispose (GObject *object)
 	if (set->holders) {
 		for (list = set->holders; list; list = list->next) {
 			g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
-				G_CALLBACK (changed_holder_cb), set);
+							      G_CALLBACK (changed_holder_cb), set);
 			g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
-				G_CALLBACK (source_changed_holder_cb), set);
+							      G_CALLBACK (validate_change_holder_cb), set);
 			g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
-				G_CALLBACK (notify_holder_cb), set);
+							      G_CALLBACK (source_changed_holder_cb), set);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
+							      G_CALLBACK (att_holder_changed_cb), set);
 			g_object_unref (G_OBJECT (list->data));
 		}
 		g_slist_free (set->holders);
 	}
+	if (set->priv->holders_hash) {
+		g_hash_table_destroy (set->priv->holders_hash);
+		set->priv->holders_hash = NULL;
+	}
 
 	/* free the nodes if there are some */
 	while (set->nodes_list)
@@ -1031,7 +1060,7 @@ compute_public_data (GdaSet *set)
 	for (list = set->holders; list; list = list->next) {
 		node = g_new0 (GdaSetNode, 1);
 		node->holder = GDA_HOLDER (list->data);
-		node->source_model = gda_holder_get_source_model (GDA_HOLDER (list->data),
+		node->source_model = gda_holder_get_source_model (node->holder,
 								  &(node->source_column));
 		if (node->source_model)
 			g_object_ref (node->source_model);
@@ -1204,56 +1233,66 @@ compute_ref_columns_index (GdaSetSource *source)
  * Adds @holder to the list of holders managed within @set.
  *
  * NOTE: if @set already has a #GdaHolder with the same ID as @holder, then @holder
- * will not be added to the set, even if @holder's type or value is not the same as the
- * on already in @set.
+ * will not be added to the set (even if @holder's type or value is not the same as the
+ * one already in @set).
+ *
+ * Returns: TRUE if @holder has been added to @set (and FALSE if it has not been added because there is another #GdaHolder
+ * with the same ID)
  */
-void
+gboolean
 gda_set_add_holder (GdaSet *set, GdaHolder *holder)
 {
-	g_return_if_fail (GDA_IS_SET (set));
-	g_return_if_fail (GDA_IS_HOLDER (holder));
+	gboolean added;
+	g_return_val_if_fail (GDA_IS_SET (set), FALSE);
+	g_return_val_if_fail (GDA_IS_HOLDER (holder), FALSE);
 
-	gda_set_real_add_holder (set, holder);
-	compute_public_data (set);
+	added = gda_set_real_add_holder (set, holder);
+	if (added)
+		compute_public_data (set);
+	return added;
 }
 
-static void
+static gboolean
 gda_set_real_add_holder (GdaSet *set, GdaHolder *holder)
 {
-	GSList *holders;
-	GdaHolder *similar = NULL;
-	const gchar *ncid;
+	GdaHolder *similar;
+	const gchar *hid;
 
 	if (g_slist_find (set->holders, holder))
-		return;
+		return FALSE;
 
 	/* 
 	 * try to find a similar holder in the set->holders:
 	 * a holder B is similar to a holder A if it has the same ID
 	 */
-	ncid = gda_holder_get_id (holder);
-	for (holders = set->holders; ncid && holders && !similar; holders = holders->next) {
-		const gchar *cid;
-		cid = gda_holder_get_id (GDA_HOLDER (holders->data));
-		if (cid && !strcmp (cid, ncid))
-			similar = GDA_HOLDER (holders->data);
+	hid = gda_holder_get_id (holder);
+	if (!hid) {
+		g_warning (_("GdaHolder needs to have an ID"));
+		return FALSE;
 	}
-	
+	g_return_val_if_fail (hid, FALSE);
+	similar = (GdaHolder*) g_hash_table_lookup (set->priv->holders_hash, hid);
 	if (!similar) {
 		/* really add @holder to the set */
 		set->holders = g_slist_append (set->holders, holder);
+		g_hash_table_insert (set->priv->holders_hash, (gchar*) hid, holder);
 		g_object_ref (holder);
 		g_signal_connect (G_OBJECT (holder), "changed",
 				  G_CALLBACK (changed_holder_cb), set);
-		g_signal_connect (G_OBJECT (holder), "source_changed",
+		g_signal_connect (G_OBJECT (holder), "validate-change",
+				  G_CALLBACK (validate_change_holder_cb), set);
+		g_signal_connect (G_OBJECT (holder), "source-changed",
 				  G_CALLBACK (source_changed_holder_cb), set);
-		g_signal_connect (G_OBJECT (holder), "notify",
-				  G_CALLBACK (notify_holder_cb), set);
+		g_signal_connect (G_OBJECT (holder), "attribute-changed",
+				  G_CALLBACK (att_holder_changed_cb), set);
+		return TRUE;
 	}
+	else {
 #ifdef GDA_DEBUG_NO
-	else 
 		g_print ("In Set %p, Holder %p and %p are similar, keeping %p only\n", set, similar, holder, similar);
 #endif
+		return FALSE;
+	}
 }
 
 
@@ -1307,32 +1346,46 @@ set_remove_source (GdaSet *set, GdaSetSource *source)
 /**
  * gda_set_is_valid
  * @set: a #GdaSet object
+ * @error: a place to store validation errors, or %NULL
  *
- * Tells if all the set's holders have valid data
+ * This method tells if all @set's #GdaHolder objects are valid, and if
+ * they represent a valid combination of values, as defined by rules
+ * external to Libgda: the "validate-set" signal is emitted and if none of the signal handlers return an
+ * error, then the returned value is TRUE, otherwise the return value is FALSE as soon as a signal handler
+ * returns an error.
  *
  * Returns: TRUE if the set is valid
  */
 gboolean
-gda_set_is_valid (GdaSet *set)
+gda_set_is_valid (GdaSet *set, GError **error)
 {
 	GSList *holders;
-	gboolean retval = TRUE;
 
 	g_return_val_if_fail (GDA_IS_SET (set), FALSE);
 	g_return_val_if_fail (set->priv, FALSE);
 
-	for (holders = set->holders; holders && retval; holders = g_slist_next (holders)) {
-		if (!gda_holder_is_valid (GDA_HOLDER (holders->data))) 
-			retval = FALSE;
-		
-#ifdef GDA_DEBUG_NO
-		g_print ("== HOLDER %p: valid= %d, value=%s\n", holders->data, gda_holder_is_valid (GDA_HOLDER (holders->data)),
-			 gda_holder_get_value (GDA_HOLDER (holders->data)) ?
-			 gda_value_stringify (gda_holder_get_value (GDA_HOLDER (holders->data))) : "Null");
-#endif
+	for (holders = set->holders; holders; holders = holders->next) {
+		if (!gda_holder_is_valid ((GdaHolder*) holders->data)) {
+			g_set_error (error, GDA_SET_ERROR, GDA_SET_INVALID_ERROR,
+				     "%s", _("One or more values are invalid"));
+			return FALSE;
+		}
 	}
 
-	return retval;
+	/* signal the holder validate-set */
+	GError *lerror = NULL;
+#ifdef GDA_DEBUG_signal
+	g_print (">> 'VALIDATE_SET' from %s\n", __FUNCTION__);
+#endif
+	g_signal_emit (G_OBJECT (set), gda_set_signals[VALIDATE_SET], 0, &lerror);
+#ifdef GDA_DEBUG_signal
+	g_print ("<< 'VALIDATE_SET' from %s\n", __FUNCTION__);
+#endif
+	if (lerror) {
+		g_propagate_error (error, lerror);
+		return FALSE;
+	}
+	return TRUE;
 }
 
 /**
@@ -1347,20 +1400,10 @@ gda_set_is_valid (GdaSet *set)
 GdaHolder *
 gda_set_get_holder (GdaSet *set, const gchar *holder_id)
 {
-	GdaHolder *holder = NULL;
-	GSList *list;
-	gchar *hid;
-
 	g_return_val_if_fail (GDA_IS_SET (set), NULL);
 	g_return_val_if_fail (set->priv, NULL);
 
-	for (list = set->holders; list && !holder; list = list->next) {
-		hid = (gchar *) gda_holder_get_id (GDA_HOLDER (list->data));
-		if (hid && !strcmp (hid, holder_id))
-			holder = GDA_HOLDER (list->data);
-	}
-
-	return holder;
+	return (GdaHolder *) g_hash_table_lookup (set->priv->holders_hash, holder_id);
 }
 
 
