@@ -1,5 +1,5 @@
 /* GDA common library
- * Copyright (C) 1998 - 2007 The GNOME Foundation.
+ * Copyright (C) 1998 - 2008 The GNOME Foundation.
  *
  * AUTHORS:
  *	Michael Lausch <michael@lausch.at>
@@ -28,20 +28,18 @@
 #include <string.h>
 #include <stdlib.h>
 #include <time.h>
-#include <glib/gdate.h>
-#include <glib/gmessages.h>
-#include <glib/gstrfuncs.h>
-#include <glib/gstring.h>
-#include <glib/gi18n-lib.h>
 #include <libgda/gda-value.h>
 #include <libgda/gda-blob-op.h>
 #include <libgda/gda-util.h>
 #include <libxml/parser.h>
 #include <libxml/tree.h>
+#define __GDA_INTERNAL__
+#include "dir-blob-op.h"
 
-#define l_g_value_unset(val) if G_IS_VALUE (val) g_value_unset (val)
-
-
+#define l_g_value_unset(val) G_STMT_START{ if (G_IS_VALUE (val)) g_value_unset (val); }G_STMT_END
+#ifdef G_OS_WIN32
+#define bcmp(s1, s2, n) memcmp ((s1), (s2), (n))
+#endif
 
 static gboolean
 set_from_string (GValue *value, const gchar *as_string)
@@ -60,60 +58,40 @@ set_from_string (GValue *value, const gchar *as_string)
 	}
 
 	type = G_VALUE_TYPE (value);
-	gda_value_reset_with_type (value, type);
-
-	if (g_value_type_transformable (G_TYPE_STRING, type)) {
-		/* use the GLib type transformation function */
-		GValue *string;
-
-                string = g_new0 (GValue, 1);
-                g_value_init (string, G_TYPE_STRING);
-                g_value_set_string (string, as_string);
-
-                g_value_transform (string, value);
-                gda_value_free (string);
-
-                return TRUE;
-	}
+	g_value_reset (value);
 
 	/* custom transform function */
 	retval = FALSE;
 	if (type == G_TYPE_BOOLEAN) {
-		if (g_ascii_strcasecmp (as_string, "true") == 0) {
+		if (((as_string[0] == 't') || (as_string[0] == 'T')) &&
+		    ((as_string[1] == 'r') || (as_string[1] == 'R')) &&
+		    ((as_string[2] == 'u') || (as_string[2] == 'U')) &&
+		    ((as_string[3] == 'e') || (as_string[3] == 'E'))) {
 			g_value_set_boolean (value, TRUE);
 			retval = TRUE;
 		}
-		else {
-			if (g_ascii_strcasecmp (as_string, "false") == 0) {
-				g_value_set_boolean (value, FALSE);
-				retval = TRUE;
-			}
+		else if (((as_string[0] == 'f') || (as_string[0] == 'F')) &&
+			 ((as_string[1] == 'a') || (as_string[1] == 'A')) &&
+			 ((as_string[2] == 'l') || (as_string[2] == 'L')) &&
+			 ((as_string[3] == 's') || (as_string[3] == 'S')) &&
+			 ((as_string[4] == 'e') || (as_string[4] == 'E'))) {
+			g_value_set_boolean (value, FALSE);
+			retval = TRUE;
 		}
 	}
-	else if (type == GDA_TYPE_BINARY) {
-		GdaBinary binary;
-		retval = gda_string_to_binary (as_string, &binary);
-		if (retval)
-			gda_value_set_binary (value, &binary);
-	}
-	else if (type == GDA_TYPE_BLOB) {
-		GdaBlob blob;
-		retval = gda_string_to_blob (as_string, &blob);
-		if (retval)
-			gda_value_set_blob (value, &blob);
-	}
 	else if (type == G_TYPE_INT64) {
-		/* Use g_strtod instead of strtoll */
-		dvalue = g_strtod (as_string, endptr);
+		gint64 i64;
+		i64 = g_ascii_strtoll (as_string, endptr, 10);
 		if (*as_string != '\0' && **endptr == '\0'){
-			g_value_set_int64 (value, (gint64) dvalue);
+			g_value_set_int64 (value, i64);
 			retval = TRUE;
 		}
 	}
 	else if (type == G_TYPE_UINT64) {
- 	        dvalue = g_strtod (as_string, endptr);
-                if (*as_string!=0 && **endptr==0) {
-			g_value_set_uint64(value,(guint64)dvalue);
+		guint64 ui64;
+		ui64 = g_ascii_strtoull (as_string, endptr, 10);
+		if (*as_string != '\0' && **endptr == '\0'){
+			g_value_set_uint64 (value, ui64);
 			retval = TRUE;
                 }
 	}
@@ -146,28 +124,28 @@ set_from_string (GValue *value, const gchar *as_string)
 		}
 	}
 	else if (type == G_TYPE_CHAR) {
-		lvalue = strtol(as_string, endptr, 10);
+		lvalue = strtol (as_string, endptr, 10);
 		if (*as_string!=0 && **endptr==0) {
 			g_value_set_char(value,(gchar)lvalue);
 			retval = TRUE;
 		}
 	}
 	else if (type == G_TYPE_UCHAR) {
-		ulvalue = strtoul(as_string,endptr, 10);
+		ulvalue = strtoul (as_string,endptr, 10);
 		if (*as_string!=0 && **endptr==0) {
 			g_value_set_uchar(value,(guchar)ulvalue);
 			retval = TRUE;
 		}
 	}
 	else if (type == G_TYPE_FLOAT) {
-		dvalue = g_strtod (as_string, endptr);
+		dvalue = g_ascii_strtod (as_string, endptr);
 		if (*as_string != '\0' && **endptr == '\0'){
 			g_value_set_float (value, (gfloat) dvalue);
 			retval = TRUE;
 		}
 	}
 	else if (type == G_TYPE_DOUBLE) {
-		dvalue = g_strtod (as_string, endptr);
+		dvalue = g_ascii_strtod (as_string, endptr);
 		if (*as_string != '\0' && **endptr == '\0'){
 			g_value_set_double (value, dvalue);
 			retval = TRUE;
@@ -175,7 +153,7 @@ set_from_string (GValue *value, const gchar *as_string)
 	}
 	else if (type == GDA_TYPE_NUMERIC) {
 		GdaNumeric numeric;
-		/* FIXME: what test whould i do for numeric? */
+		/* FIXME: what test whould i do for numeric? Use GMP (http://gmplib.org/) ?*/
 		numeric.number = g_strdup (as_string);
 		numeric.precision = 0; /* FIXME */
 		numeric.width = 0; /* FIXME */
@@ -186,31 +164,85 @@ set_from_string (GValue *value, const gchar *as_string)
 	else if (type == G_TYPE_DATE) {
 		GDate *gdate;
 		gdate = g_date_new ();
-		g_date_set_parse (gdate, as_string);
-		if (g_date_valid (gdate)) {
+
+		if (gda_parse_iso8601_date (gdate, as_string)) {
 			g_value_take_boxed (value, gdate);
 			retval = TRUE;
 		}
 		else
 			g_date_free (gdate);
 	}
+	else if (type == GDA_TYPE_TIME) {
+		GdaTime timegda;
+		if (gda_parse_iso8601_time (&timegda, as_string)) {
+			gda_value_set_time (value, &timegda);
+			retval = TRUE;
+		}
+	}
+	else if (type == GDA_TYPE_TIMESTAMP) {
+		GdaTimestamp timestamp;
+		if (gda_parse_iso8601_timestamp (&timestamp, as_string)) {
+			gda_value_set_timestamp (value, &timestamp);
+			retval = TRUE;
+		}
+	}
 	else if (type == GDA_TYPE_NULL) {
 		gda_value_set_null (value);
 		retval = TRUE;
 	}
-	else if (type == G_TYPE_ULONG) {
-		if (gda_g_type_from_string (as_string) != 0) 
-			g_value_set_ulong (value, gda_g_type_from_string (as_string));
-		else {
-			gulong ulvalue;
-
-			ulvalue = strtoul (as_string, endptr, 10);
-			if (*as_string!=0 && **endptr==0) {
-				g_value_set_ulong (value, ulvalue);
-				retval = TRUE;
-			}
+	else if (type == G_TYPE_GTYPE) {
+		if (gda_g_type_from_string (as_string) != 0) {
+			g_value_set_gtype (value, gda_g_type_from_string (as_string));
+			retval = TRUE;
 		}
 	}
+	else if (type == G_TYPE_ULONG)
+	{
+		gulong ulvalue;
+		
+		ulvalue = strtoul (as_string, endptr, 10);
+		if (*as_string!=0 && **endptr==0) {
+			g_value_set_ulong (value, ulvalue);
+			retval = TRUE;
+		}
+	}
+	else if (type == G_TYPE_LONG)
+	{
+		glong lvalue;
+		
+		lvalue = strtol (as_string, endptr, 10);
+		if (*as_string!=0 && **endptr==0) {
+			g_value_set_long (value, lvalue);
+			retval = TRUE;
+		}
+	}
+	else if (type == GDA_TYPE_BINARY) {
+		GdaBinary binary;
+		retval = gda_string_to_binary (as_string, &binary);
+		if (retval)
+			gda_value_set_binary (value, &binary);
+	}
+	else if (type == GDA_TYPE_BLOB) {
+		GdaBlob blob;
+		retval = gda_string_to_blob (as_string, &blob);
+		if (retval)
+			gda_value_set_blob (value, &blob);
+	}
+
+
+	if (!retval && g_value_type_transformable (G_TYPE_STRING, type)) {
+		/* use the GLib type transformation function */
+		GValue *string;
+
+                string = g_new0 (GValue, 1);
+                g_value_init (string, G_TYPE_STRING);
+                g_value_set_string (string, as_string);
+
+                g_value_transform (string, value);
+                gda_value_free (string);
+
+                retval = TRUE;
+	}
 
 	return retval;
 }
@@ -310,6 +342,7 @@ gda_binary_free (gpointer boxed)
 	g_return_if_fail (binary);
 		
 	g_free (binary->data);
+	g_free (binary);
 }
 
 /* 
@@ -509,16 +542,17 @@ gda_geometricpoint_copy (gpointer boxed)
 	
 	g_return_val_if_fail( val, NULL);
 		
-	copy = g_new0(GdaGeometricPoint, 1);
+	copy = g_new0 (GdaGeometricPoint, 1);
 	copy->x = val->x;
 	copy->y = val->y;
 
 	return copy;
 }
 
-void gda_geometricpoint_free (gpointer boxed)
+void
+gda_geometricpoint_free (gpointer boxed)
 {
-	
+	g_free (boxed);
 }
 
 
@@ -602,10 +636,11 @@ gda_value_list_copy (gpointer boxed)
 	return list;
 }
 
-void gda_value_list_free (gpointer boxed)
+void
+gda_value_list_free (gpointer boxed)
 {
 	GList *l = (GList*) boxed;
-	g_list_free(l);
+	g_list_free (l);
 }
 
 
@@ -735,7 +770,6 @@ gda_numeric_free (gpointer boxed)
 	g_return_if_fail (numeric);
 
 	g_free (numeric->number);
-
 	g_free (numeric);
 }
 
@@ -890,9 +924,9 @@ gda_time_copy (gpointer boxed)
 	GdaTime *src = (GdaTime*) boxed;
 	GdaTime *copy = NULL;
 	
-	g_return_val_if_fail(src, NULL);
+	g_return_val_if_fail (src, NULL);
 	
-	copy = g_new0(GdaTime, 1);
+	copy = g_new0 (GdaTime, 1);
 	copy->hour = src->hour;
 	copy->minute = src->minute;
 	copy->second = src->second;
@@ -902,9 +936,10 @@ gda_time_copy (gpointer boxed)
 	return copy;
 }
 
-void gda_time_free (gpointer boxed)
+void
+gda_time_free (gpointer boxed)
 {
-	
+	g_free (boxed);
 }
 
 
@@ -1003,7 +1038,7 @@ gda_timestamp_copy (gpointer boxed)
 	
 	g_return_val_if_fail(src, NULL);
 	
-	copy = g_new0(GdaTimestamp, 1);
+	copy = g_new0 (GdaTimestamp, 1);
 	copy->year = src->year;
 	copy->month = src->month;
 	copy->day = src->day;
@@ -1016,9 +1051,10 @@ gda_timestamp_copy (gpointer boxed)
 	return copy;
 }
 
-void gda_timestamp_free (gpointer boxed)
+void
+gda_timestamp_free (gpointer boxed)
 {
-	
+	g_free (boxed);
 }
 
 
@@ -1075,7 +1111,7 @@ gda_value_new_binary (const guchar *val, glong size)
  * @val: value to set for the new #GValue.
  * @size: the size of the memory pool pointer to by @val.
  *
- * Makes a new #GValue of type #GDA_TYPE_BLOB with value @val.
+ * Makes a new #GValue of type #GDA_TYPE_BLOB with the data contained by @val.
  *
  * Returns: the newly created #GValue.
  */
@@ -1083,20 +1119,44 @@ GValue *
 gda_value_new_blob (const guchar *val, glong size)
 {
 	GValue *value;
-	GdaBlob blob;
+	GdaBlob *blob;
 	GdaBinary *bin;
 
+	blob = g_new0 (GdaBlob, 1);
 	bin = (GdaBinary*)(&blob);
-
-        /* We use the const on the function parameter to make this clearer, 
-	 * but it would be awkward to keep the const in the struct.
-         */
-        bin->data = (guchar*)val;
+	bin->data = g_new (guchar, size);
+        memcpy ((gpointer) bin->data, (gpointer) val, size);
         bin->binary_length = size;
-	blob.op = NULL;
+	blob->op = NULL;
 
         value = g_new0 (GValue, 1);
-        gda_value_set_blob (value, &blob);
+	g_value_init (value, GDA_TYPE_BLOB);
+        g_value_take_boxed (value, blob);
+
+        return value;
+}
+
+/**
+ * gda_value_new_blob
+ * @val: value to set for the new #GValue.
+ * @size: the size of the memory pool pointer to by @val.
+ *
+ * Makes a new #GValue of type #GDA_TYPE_BLOB with the data contained by @val.
+ *
+ * Returns: the newly created #GValue.
+ */
+GValue *
+gda_value_new_blob_from_file (const gchar *filename)
+{
+	GValue *value;
+	GdaBlob *blob;
+
+	blob = g_new0 (GdaBlob, 1);
+	blob->op = gda_dir_blob_op_new (filename);
+
+        value = g_new0 (GValue, 1);
+	g_value_init (value, GDA_TYPE_BLOB);
+        g_value_take_boxed (value, blob);
 
         return value;
 }
@@ -1140,7 +1200,12 @@ gda_value_new_timestamp_from_timet (time_t val)
  * @as_string: stringified representation of the value.
  * @type: the new value type.
  *
- * Makes a new #GValue of type @type from its string representation.
+ * Makes a new #GValue of type @type from its string representation. 
+ *
+ * For more information
+ * about the string format, see the gda_value_set_from_string() function.
+ * This function is typically used when reading configuration files or other non-user input that should be locale 
+ * independent.
  *
  * Returns: the newly created #GValue or %NULL if the string representation
  * cannot be converted to the specified @type.
@@ -1163,18 +1228,24 @@ gda_value_new_from_string (const gchar *as_string, GType type)
 
 /**
  * gda_value_new_from_xml
- * @node: a XML node representing the value.
+ * @node: an XML node representing the value.
  *
- * Creates a GValue from a XML representation of it. That XML
+ * Creates a GValue from an XML representation of it. That XML
  * node corresponds to the following string representation:
- *    &lt;value type="gdatype"&gt;value&lt;/value&gt;
+ * &lt;value type="gdatype"&gt;value&lt;/value&gt;
  *
- * Returns:  the newly created #GValue.
+ * For more information
+ * about the string format, see the gda_value_set_from_string() function.
+ * This function is typically used when reading configuration files or other non-user input that should be locale 
+ * independent.
+ *
+ * Returns: the newly created #GValue.
  */
 GValue *
 gda_value_new_from_xml (const xmlNodePtr node)
 {
 	GValue *value;
+	xmlChar *prop;
 
 	g_return_val_if_fail (node, NULL);
 
@@ -1183,12 +1254,15 @@ gda_value_new_from_xml (const xmlNodePtr node)
 		return NULL;
 
 	value = g_new0 (GValue, 1);
-	if (!gda_value_set_from_string (value,
-					(gchar*)xmlNodeGetContent (node),
-					g_type_from_name ((gchar*)xmlGetProp(node, (xmlChar*)"gdatype")))) {
+	prop = xmlGetProp (node, (xmlChar*)"gdatype");
+	if (prop && !gda_value_set_from_string (value,
+						(gchar*)xmlNodeGetContent (node),
+						gda_g_type_from_string ((gchar*) prop))) {
 		g_free (value);
 		value = NULL;
 	}
+	if (prop)
+		xmlFree (prop);
 
 	return value;
 }
@@ -1219,11 +1293,15 @@ gda_value_reset_with_type (GValue *value, GType type)
 {
 	g_return_if_fail (value);
 
-	l_g_value_unset (value);
-	if (type == GDA_TYPE_NULL || type == G_TYPE_INVALID)
-		return;
-	else
-		g_value_init (value, type);
+	if (G_IS_VALUE (value) && (G_VALUE_TYPE (value) == type))
+		g_value_reset (value);
+	else {
+		l_g_value_unset (value);
+		if (type == GDA_TYPE_NULL || type == G_TYPE_INVALID)
+			return;
+		else
+			g_value_init (value, type);
+	}
 }
 
 
@@ -1653,6 +1731,18 @@ gda_value_set_timestamp (GValue *value, const GdaTimestamp *val)
  *
  * Stores the value data from its string representation as @type.
  *
+ * The accepted formats are:
+ * <itemizedlist>
+ *   <listitem><para>G_TYPE_BOOLEAN: a caseless comparison is made with "true" or "false"</para></listitem>
+ *   <listitem><para>numerical types: C locale format (dot as a fraction separator)</para></listitem>
+ *   <listitem><para>G_TYPE_DATE: see <link linkend="gda-parse-iso8601-date">gda_parse_iso8601_date()</link></para></listitem>
+ *   <listitem><para>GDA_TYPE_TIME: see <link linkend="gda-parse-iso8601-time">gda_parse_iso8601_time()</link></para></listitem>
+ *   <listitem><para>GDA_TYPE_TIMESTAMP: see <link linkend="gda-parse-iso8601-timestamp">gda_parse_iso8601_timestamp()</link></para></listitem>
+ * </itemizedlist>
+ *
+ * This function is typically used when reading configuration files or other non-user input that should be locale 
+ * independent.
+ *
  * Returns: %TRUE if the value has been converted to @type from
  * its string representation; it not means that the value is converted 
  * successfully, just that the transformation is avairable. %FALSE otherwise.
@@ -1690,8 +1780,13 @@ gda_value_set_from_value (GValue *value, const GValue *from)
 	g_return_val_if_fail (from, FALSE);
 
 	if (G_IS_VALUE (from)) {
-		g_value_copy (from, value);
-		return g_value_type_compatible (G_VALUE_TYPE (from), G_VALUE_TYPE (value));
+		if (g_value_type_compatible (G_VALUE_TYPE (from), G_VALUE_TYPE (value))) {
+			g_value_reset (value);
+			g_value_copy (from, value);
+			return TRUE;
+		}
+		else
+			return FALSE;
 	}
 	else {
 		l_g_value_unset (value);
@@ -1746,8 +1841,8 @@ gda_value_stringify (const GValue *value)
 					return g_strdup ("0000-00-00");
 			}
 			else if (type == GDA_TYPE_LIST) {
-				GdaValueList *list;
-				GList *ptr;
+				const GdaValueList *list;
+				const GList *ptr;
 				GString *string;
 				gchar *tmp;
 
@@ -1774,11 +1869,177 @@ gda_value_stringify (const GValue *value)
 }
 	
 /**
- * gda_value_compare
+ * gda_value_differ
  * @value1: a #GValue to compare.
  * @value2: the other #GValue to be compared to @value1.
  *
- * Compares two values of the same type.
+ * Tells if two values are equal or not, by comparing memory representations. Unlike gda_value_compare(),
+ * the returned value is boolean, and gives no idea about ordering.
+ *
+ * The two values must be of the same type, with the exception that a value of any type can be
+ * compared to a GDA_TYPE_NULL value, specifically:
+ * <itemizedlist>
+ *   <listitem><para>if @value1 and @value2 are both GDA_TYPE_NULL values then the returned value is 0</para></listitem>
+ *   <listitem><para>if @value1 is a GDA_TYPE_NULL value and @value2 is of another type then the returned value is 1</para></listitem>
+ *   <listitem><para>if @value1 is of another type and @value2 is a GDA_TYPE_NULL value then the returned value is 1</para></listitem>
+ *   <listitem><para>in all other cases, @value1 and @value2 must be of the same type and their values are compared</para></listitem>
+ * </itemizedlist>
+ *
+ * Returns: a non 0 value if @value1 and @value2 differ, and 0 if they are equal
+ */
+gint
+gda_value_differ (const GValue *value1, const GValue *value2)
+{
+	GType type;
+	g_return_val_if_fail (value1 && value2, FALSE);
+
+	/* blind value comparison */
+	type = G_VALUE_TYPE (value1);
+	if (!bcmp (value1, value2, sizeof (GValue)))
+		return 0;
+
+	/* handle GDA_TYPE_NULL comparisons with other types */
+	else if (type == GDA_TYPE_NULL) {
+		if (G_VALUE_TYPE (value2) == GDA_TYPE_NULL)
+			return 0;
+		else
+			return 1;
+	}
+
+	else if (G_VALUE_TYPE (value2) == GDA_TYPE_NULL)
+		return 1;
+
+	g_return_val_if_fail (G_VALUE_TYPE (value1) == G_VALUE_TYPE (value2), 1);
+
+	/* general case */
+	if (type == GDA_TYPE_BINARY) {
+		const GdaBinary *binary1 = gda_value_get_binary (value1);
+		const GdaBinary *binary2 = gda_value_get_binary (value2);
+		if (binary1 && binary2 && (binary1->binary_length == binary2->binary_length))
+			return bcmp (binary1->data, binary2->data, binary1->binary_length);
+		else
+			return 1;
+	}
+
+	else if (type == GDA_TYPE_BLOB) {
+		const GdaBlob *blob1 = gda_value_get_blob (value1);
+		const GdaBlob *blob2 = gda_value_get_blob (value2);
+		if (blob1 && blob2 && (((GdaBinary *)blob1)->binary_length == ((GdaBinary *)blob2)->binary_length)) {
+			if (blob1->op == blob2->op)
+				return bcmp (((GdaBinary *)blob1)->data, ((GdaBinary *)blob2)->data, 
+					     ((GdaBinary *)blob1)->binary_length);
+		}
+		return 1;
+	}
+
+	else if (type == G_TYPE_DATE) {
+		GDate *d1, *d2;
+
+		d1 = (GDate *) g_value_get_boxed (value1);
+		d2 = (GDate *) g_value_get_boxed (value2);
+		if (d1 && d2)
+			return g_date_compare (d1, d2);
+		return 1;
+	}
+
+	else if (type == GDA_TYPE_GEOMETRIC_POINT) {
+		const GdaGeometricPoint *p1, *p2;
+		p1 = gda_value_get_geometric_point (value1);
+		p2 = gda_value_get_geometric_point (value2);
+		if (p1 && p2)
+			return bcmp (p1, p2, sizeof (GdaGeometricPoint));
+		return 1;
+	}
+
+	else if (type == G_TYPE_OBJECT) {
+		if (g_value_get_object (value1) == g_value_get_object (value2))
+			return 0;
+		else
+			return -1;
+	}
+
+	else if (type == GDA_TYPE_LIST) {
+		GList *l1, *l2;
+		for (l1 = (GList*) gda_value_get_list (value1), l2 = (GList*) gda_value_get_list (value2); 
+		     l1 != NULL && l2 != NULL; l1 = l1->next, l2 = l2->next){
+			if (gda_value_differ ((GValue *) l1->data, (GValue *) l2->data))
+				return 1;
+		}
+		return 0;
+	}
+
+	else if (type == GDA_TYPE_NUMERIC) {
+		const GdaNumeric *num1, *num2;
+		num1= gda_value_get_numeric (value1);
+		num2 = gda_value_get_numeric (value2);
+                if (num1 && num2)
+			return strcmp (num1->number, num2->number);
+		return 1;
+	}
+	
+	else if (type == G_TYPE_STRING)	{
+		const gchar *str1, *str2;
+		str1 = g_value_get_string (value1);
+		str2 = g_value_get_string (value2);
+		if (str1 && str2)
+			return strcmp (str1, str2);
+		return 1;
+	}
+	
+	else if (type == GDA_TYPE_TIME) {
+		const GdaTime *t1, *t2;
+		t1 = gda_value_get_time (value1);
+		t2 = gda_value_get_time (value2);
+		if (t1 && t2)
+			return bcmp (t1, t2, sizeof (GdaTime));
+		return 1;
+	}
+
+	else if (type == GDA_TYPE_TIMESTAMP) {
+		const GdaTimestamp *ts1, *ts2;
+		ts1 = gda_value_get_timestamp (value1);
+		ts2 = gda_value_get_timestamp (value2);
+		if (ts1 && ts2)
+			return bcmp (ts1, ts2, sizeof (GdaTimestamp));
+		return 1;
+	}
+
+	else if ((type == G_TYPE_INT) ||
+		 (type == G_TYPE_UINT) ||
+		 (type == G_TYPE_INT64) ||
+		 (type == G_TYPE_UINT64) ||
+		 (type == GDA_TYPE_SHORT) ||
+		 (type == GDA_TYPE_USHORT) ||
+		 (type == G_TYPE_FLOAT) ||
+		 (type == G_TYPE_DOUBLE) ||
+		 (type == G_TYPE_BOOLEAN) ||
+		 (type == G_TYPE_CHAR) ||
+		 (type == G_TYPE_UCHAR) ||
+		 (type == G_TYPE_LONG) ||
+		 (type == G_TYPE_ULONG) ||
+		 (type == G_TYPE_GTYPE))
+		/* values here ARE different because otherwise the bcmp() at the beginning would
+		 * already have retruned */
+		return 1;
+
+	g_warning ("%s() cannot handle values of type %s", __FUNCTION__, g_type_name (G_VALUE_TYPE (value1)));
+
+	return 1;
+}
+
+/**
+ * gda_value_compare
+ * @value1: a #GValue to compare (not %NULL)
+ * @value2: the other #GValue to be compared to @value1 (not %NULL)
+ *
+ * Compares two values of the same type, with the exception that a value of any type can be
+ * compared to a GDA_TYPE_NULL value, specifically:
+ * <itemizedlist>
+ *   <listitem><para>if @value1 and @value2 are both GDA_TYPE_NULL values then the returned value is 0</para></listitem>
+ *   <listitem><para>if @value1 is a GDA_TYPE_NULL value and @value2 is of another type then the returned value is -1</para></listitem>
+ *   <listitem><para>if @value1 is of another type and @value2 is a GDA_TYPE_NULL value then the returned value is 1</para></listitem>
+ *   <listitem><para>in all other cases, @value1 and @value2 must be of the same type and their values are compared</para></listitem>
+ * </itemizedlist>
  *
  * Returns: if both values have the same type, returns 0 if both contain
  * the same value, an integer less than 0 if @value1 is less than @value2 or
@@ -1792,20 +2053,37 @@ gda_value_compare (const GValue *value1, const GValue *value2)
 	GType type;
 
 	g_return_val_if_fail (value1 && value2, -1);
-	g_return_val_if_fail (G_VALUE_TYPE (value1) == G_VALUE_TYPE (value2), -1);
 
-	type = G_VALUE_TYPE(value1);
+	type = G_VALUE_TYPE (value1);
 	
 	if (value1 == value2)
 		return 0;
 
-	else if (type == G_TYPE_INT64) 
-		return (g_value_get_int64 (value1) > g_value_get_int64 (value2)) ? 1 : 
-			((g_value_get_int64 (value1) == g_value_get_int64 (value2)) ? 0 : -1);
+	/* handle GDA_TYPE_NULL comparisons with other types */
+	else if (type == GDA_TYPE_NULL) {
+		if (G_VALUE_TYPE (value2) == GDA_TYPE_NULL)
+			return 0;
+		else
+			return -1;
+	}
+
+	else if (G_VALUE_TYPE (value2) == GDA_TYPE_NULL)
+		return 1;
+
+	/* general case */
+	g_return_val_if_fail (G_VALUE_TYPE (value1) == G_VALUE_TYPE (value2), -1);
+
+	if (type == G_TYPE_INT64) {
+		gint64 i1 = g_value_get_int64 (value1);
+		gint64 i2 = g_value_get_int64 (value2);
+		return (i1 > i2) ? 1 : ((i1 == i2) ? 0 : -1);
+	}
 		
-	else if (type == G_TYPE_UINT64)
-		return (g_value_get_uint64 (value1) > g_value_get_uint64 (value2)) ? 1 : 
-			((g_value_get_uint64 (value1) == g_value_get_uint64 (value2)) ? 0 : -1);
+	else if (type == G_TYPE_UINT64) {
+		guint64 i1 = g_value_get_uint64 (value1);
+		guint64 i2 = g_value_get_uint64 (value2);
+		return (i1 > i2) ? 1 : ((i1 == i2) ? 0 : -1);
+	}
 
 	else if (type == GDA_TYPE_BINARY) {
 		const GdaBinary *binary1 = gda_value_get_binary (value1);
@@ -1864,10 +2142,19 @@ gda_value_compare (const GValue *value1, const GValue *value2)
 			return (v1 > v2) ? 1 : -1;
 	}
 
-	else if (type == GDA_TYPE_GEOMETRIC_POINT)
-		return memcmp (gda_value_get_geometric_point(value1) , 
-			       gda_value_get_geometric_point(value2),
-			       sizeof (GdaGeometricPoint));
+	else if (type == GDA_TYPE_GEOMETRIC_POINT) {
+		const GdaGeometricPoint *p1, *p2;
+		p1 = gda_value_get_geometric_point (value1);
+		p2 = gda_value_get_geometric_point (value2);
+		if (p1 && p2)
+			return memcmp (p1, p2, sizeof (GdaGeometricPoint));
+		else if (p1)
+			return 1;
+		else if (p2)
+			return -1;
+		else
+			return 0;
+	}
 
 	else if (type == G_TYPE_OBJECT) {
 		if (g_value_get_object (value1) == g_value_get_object (value2))
@@ -1877,8 +2164,7 @@ gda_value_compare (const GValue *value1, const GValue *value2)
 	}
 
 	else if (type == G_TYPE_INT)
-		return (g_value_get_int (value1) > g_value_get_int (value2)) ? 1 : 
-			((g_value_get_int (value1) == g_value_get_int (value2)) ? 0 : -1);
+		return g_value_get_int (value1) - g_value_get_int (value2);
 
 	else if (type == GDA_TYPE_LIST) {
 		retval = 0;
@@ -1914,25 +2200,35 @@ gda_value_compare (const GValue *value1, const GValue *value2)
 		return retval;
 	}
 
-	else if (type == G_TYPE_FLOAT)
-		return (g_value_get_float (value1) > g_value_get_float (value2)) ? 1 : 
-			((g_value_get_float (value1) == g_value_get_float (value2)) ? 0 : -1);
+	else if (type == G_TYPE_FLOAT) {
+		gfloat f1 = g_value_get_float (value1);
+		gfloat f2 = g_value_get_float (value2);
+		return (f1 > f2) ? 1 : ((f1 == f2) ? 0 : -1);
+	}
 
-	else if (type == GDA_TYPE_SHORT)
-		return (gda_value_get_short (value1) > gda_value_get_short (value2)) ? 1 : 
-			((gda_value_get_short (value1) == gda_value_get_short (value2)) ? 0 : -1);
+	else if (type == GDA_TYPE_SHORT) {
+		gshort i1 = gda_value_get_short (value1);
+		gshort i2 = gda_value_get_short (value2);
+		return (i1 > i2) ? 1 : ((i1 == i2) ? 0 : -1);
+	}
 
-	else if (type == G_TYPE_ULONG)
-		return (g_value_get_ulong (value1) > g_value_get_ulong (value2)) ? 1 : 
-			((g_value_get_ulong (value1) == g_value_get_ulong (value2)) ? 0 : -1);
+	else if (type == G_TYPE_ULONG) {
+		gulong i1 = g_value_get_ulong (value1);
+		gulong i2 = g_value_get_ulong (value2);
+		return (i1 > i2) ? 1 : ((i1 == i2) ? 0 : -1);
+	}
 
-	else if (type == G_TYPE_LONG)
-		return (g_value_get_long (value1) > g_value_get_long (value2)) ? 1 : 
-			((g_value_get_long (value1) == g_value_get_long (value2)) ? 0 : -1);
+	else if (type == G_TYPE_LONG) {
+		glong i1 = g_value_get_long (value1);
+		glong i2 = g_value_get_long (value2);
+		return (i1 > i2) ? 1 : ((i1 == i2) ? 0 : -1);
+	}
 	
-	else if (type == GDA_TYPE_USHORT)
-		return (gda_value_get_ushort (value1) > gda_value_get_ushort (value2)) ? 1 : 
-			((gda_value_get_ushort (value1) == gda_value_get_ushort (value2)) ? 0 : -1);
+	else if (type == GDA_TYPE_USHORT) {
+		gushort i1 = gda_value_get_ushort (value1);
+		gushort i2 = gda_value_get_ushort (value2);
+		return (i1 > i2) ? 1 : ((i1 == i2) ? 0 : -1);
+	}
 	
 	else if (type == G_TYPE_STRING)	{
 		const gchar *str1, *str2;
@@ -1954,69 +2250,63 @@ gda_value_compare (const GValue *value1, const GValue *value2)
 		return retval;
 	}
 	
-	else if (type == GDA_TYPE_TIME)
-		return memcmp (gda_value_get_time(value1), gda_value_get_time(value2),
-			       sizeof (GdaTime));
+	else if (type == GDA_TYPE_TIME) {
+		const GdaTime *t1, *t2;
+		t1 = gda_value_get_time (value1);
+		t2 = gda_value_get_time (value2);
+		if (t1 && t2)
+			return memcmp (t1, t2, sizeof (GdaTime));
+		else if (t1)
+			return 1;
+		else if (t2)
+			return -1;
+		else
+			return 0;
+	}
 
-	else if (type == GDA_TYPE_TIMESTAMP)
-		return memcmp (gda_value_get_timestamp(value1), 
-			       gda_value_get_timestamp(value2),
-			       sizeof (GdaTimestamp));
-	
-	else if (type == G_TYPE_CHAR)
-		return (g_value_get_char (value1) > g_value_get_char (value2)) ? 1 : 
-			((g_value_get_char (value1) == g_value_get_char (value2)) ? 0 : -1);
+	else if (type == GDA_TYPE_TIMESTAMP) {
+		const GdaTimestamp *ts1, *ts2;
+		ts1 = gda_value_get_timestamp (value1);
+		ts2 = gda_value_get_timestamp (value2);
+		if (ts1 && ts2)
+			return memcmp (ts1, ts2, sizeof (GdaTimestamp));
+		else if (ts1)
+			return 1;
+		else if (ts2)
+			return -1;
+		else
+			return 0;
+	}
 
-	else if (type == G_TYPE_UCHAR)
-		return (g_value_get_uchar (value1) > g_value_get_uchar (value2)) ? 1 : 
-			((g_value_get_uchar (value1) == g_value_get_uchar (value2)) ? 0 : -1);
+	else if (type == G_TYPE_CHAR) {
+		gchar c1 = g_value_get_char (value1);
+		gchar c2 = g_value_get_char (value2);
+		return (c1 > c2) ? 1 : ((c1 == c2) ? 0 : -1);
+	}
 
-	else if (type == G_TYPE_UINT)
-		return (g_value_get_uint (value1) > g_value_get_uint (value2)) ? 1 : 
-			((g_value_get_uint (value1) == g_value_get_uint (value2)) ? 0 : -1);
+	else if (type == G_TYPE_UCHAR) {
+		guchar c1 = g_value_get_uchar (value1);
+		guchar c2 = g_value_get_uchar (value2);
+		return (c1 > c2) ? 1 : ((c1 == c2) ? 0 : -1);
+	}
 
+	else if (type == G_TYPE_UINT) {
+		guint i1 = g_value_get_uint (value1);
+		guint i2 = g_value_get_uint (value2);
+		return (i1 > i2) ? 1 : ((i1 == i2) ? 0 : -1);
+	}
+
+	else if (type == G_TYPE_GTYPE) {
+		GType t1 = g_value_get_gtype (value1);
+		GType t2 = g_value_get_gtype (value2);
+		return (t1 > t2) ? 1 : ((t1 == t2) ? 0: -1);
+	}
 
 	g_warning ("%s() cannot handle values of type %s", __FUNCTION__, g_type_name (G_VALUE_TYPE (value1)));
 
 	return 0;
 }
 
-#define _value_is_null(x) (!(x) || (G_VALUE_TYPE (x) == GDA_TYPE_NULL) || \
-			   ((G_VALUE_TYPE (x) == G_TYPE_STRING) && !g_value_get_string (x)))
-
-/**
- * gda_value_compare_ext
- * @value1: a #GValue to compare.
- * @value2: the other #GValue to be compared to @value1.
- *
- * Like gda_value_compare(), compares two values of the same type, except that NULL values and values
- * of type GDA_TYPE_NULL are considered equals
- *
- * Returns: 0 if both contain the same value, an integer less than 0 if @value1 is less than @value2 or
- * an integer greater than 0 if @value1 is greater than @value2.
- */
-gint
-gda_value_compare_ext (const GValue *value1, const GValue *value2)
-{
-	if (value1 == value2)
-		return 0;
-
-	if (_value_is_null (value1)) {
-		/* value1 represents a NULL value */
-		if (_value_is_null (value2))
-			return 0;
-		else
-			return 1;
-	}
-	else {
-		/* value1 does not represents a NULL value */
-		if (_value_is_null (value2))
-			return -1;
-		else
-			return gda_value_compare (value1, value2);
-	}
-}
-
 /*
  * to_string
  * 
@@ -2047,7 +2337,7 @@ to_string (const GValue *value)
  * gda_value_to_xml
  * @value: a #GValue.
  *
- * Serializes the given #GValue to a XML node string.
+ * Serializes the given #GValue to an XML node string.
  *
  * Returns: the XML node. Once not needed anymore, you should free it.
  */
@@ -2363,5 +2653,8 @@ gda_blob_to_string (GdaBlob *blob, guint maxlen)
 gboolean
 gda_string_to_blob (const gchar *str, GdaBlob *blob)
 {
-	return gda_string_to_binary (str, (GdaBinary*) blob);
+	gboolean retval;
+	retval = gda_string_to_binary (str, (GdaBinary*) blob);
+	blob->op = NULL;
+	return retval;
 }
