@@ -1,6 +1,6 @@
 /* gda-handler-bin.h
  *
- * Copyright (C) 2005 - 2007 Vivien Malerba
+ * Copyright (C) 2005 - 2008 Vivien Malerba
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -21,7 +21,7 @@
 #ifndef __GDA_HANDLER_BIN__
 #define __GDA_HANDLER_BIN__
 
-#include <libgda/gda-object.h>
+#include <glib-object.h>
 #include <libgda/gda-data-handler.h>
 
 G_BEGIN_DECLS
@@ -31,16 +31,14 @@ G_BEGIN_DECLS
 #define GDA_HANDLER_BIN_CLASS(klass)  G_TYPE_CHECK_CLASS_CAST (klass, gda_handler_bin_get_type (), GdaHandlerBinClass)
 #define GDA_IS_HANDLER_BIN(obj)       G_TYPE_CHECK_INSTANCE_TYPE (obj, gda_handler_bin_get_type ())
 
-
 typedef struct _GdaHandlerBin      GdaHandlerBin;
 typedef struct _GdaHandlerBinClass GdaHandlerBinClass;
 typedef struct _GdaHandlerBinPriv  GdaHandlerBinPriv;
 
-
 /* struct for the object's data */
 struct _GdaHandlerBin
 {
-	GdaObject           object;
+	GObject             object;
 
 	GdaHandlerBinPriv  *priv;
 };
@@ -48,11 +46,11 @@ struct _GdaHandlerBin
 /* struct for the object's class */
 struct _GdaHandlerBinClass
 {
-	GdaObjectClass      parent_class;
+	GObjectClass        parent_class;
 };
 
 
-GType           gda_handler_bin_get_type      (void);
+GType           gda_handler_bin_get_type      (void) G_GNUC_CONST;
 GdaDataHandler *gda_handler_bin_new           (void);
 
 G_END_DECLS
