@@ -29,10 +29,10 @@ main (int argc, char **argv)
 	number_failed = prov_test_common_setup ();
 
 	if (cnc) {
-		number_failed += prov_test_common_create_tables_sql ();
 		number_failed += prov_test_common_check_meta ();
 		number_failed += prov_test_common_load_data ();
 		number_failed += prov_test_common_check_cursor_models ();
+		number_failed += prov_test_common_check_data_select ();
 		number_failed += prov_test_common_clean ();
 	}
 
